@@ -8,12 +8,15 @@
  */
 
 #import <Foundation/Foundation.h>
-#import "osx_ruby.h"
-#import "osx_intern.h"
+#import <ruby.h>
 #import "BridgeSupport.h"
 #import <dlfcn.h>
+#ifdef HAVE_RUBY_ST_H
+#import <ruby/st.h>
+#else
 #import <st.h>
 #import <env.h>
+#endif
 #import <objc/objc-class.h>
 #import <objc/objc-runtime.h>
 #import "ocdata_conv.h"
@@ -24,7 +27,6 @@
 #import "RBClassUtils.h"
 #import "mdl_osxobjc.h"
 #import "ocexception.h"
-#import "objc_compat.h"
 
 #define BS_LOG(fmt, args...) DLOG("BRIDGE", fmt, ##args)
 
@@ -279,7 +281,7 @@ undecorate_encoding(const char *src, char *dest, size_t dest_len, struct bsStruc
     p_src = pos + 1;
     pos = strchr(p_src, '"');
     if (pos == NULL) {
-      BS_LOG("Can't find the end of field delimiter starting at %d", p_src - src);
+      BS_LOG("Can't find the end of field delimiter starting at %ld", (long)(p_src - src));
       goto bails; 
     }
     if (field != NULL) {
@@ -329,7 +331,7 @@ undecorate_encoding(const char *src, char *dest, size_t dest_len, struct bsStruc
       }
 
       if (ok == NO) {
-        BS_LOG("Can't find the field encoding starting at %d", p_src - src);
+        BS_LOG("Can't find the field encoding starting at %ld", (long)(p_src - src));
         goto bails;
       }
 
@@ -386,12 +388,28 @@ bails:
   return NO;
 }
 
+/*
+ * Returns type encoding.
+ * @return [String]
+ * @example
+ *     OSX::NSRange.encoding
+ *     => "{_NSRange=QQ}"
+ */
 static VALUE
 rb_bs_boxed_get_encoding (VALUE rcv)
 {
-  return rb_ivar_get(rcv, ivarEncodingID);  
+  return rb_ivar_get(rcv, ivarEncodingID);
 }
 
+/*
+ * Returns allocation size.
+ * @return [Number]
+ * @example
+ *     OSX::NSRect.size
+ *     => 32
+ *     OSX::NSSize.size
+ *     => 16
+ */
 static VALUE
 rb_bs_boxed_get_size (VALUE rcv)
 {
@@ -402,6 +420,15 @@ rb_bs_boxed_get_size (VALUE rcv)
   return LONG2NUM(bs_boxed_size(boxed));
 }
 
+/*
+ * Returns field names.
+ * @return [Array]
+ * @example
+ *      OSX::NSRect.fields
+ *      => [:origin, :size]
+ *      OSX::NSSize.fields
+ *      => [:width, :height]
+ */
 static VALUE
 rb_bs_boxed_get_fields (VALUE rcv)
 {
@@ -425,6 +452,14 @@ rb_bs_boxed_get_fields (VALUE rcv)
   return ary;
 }
 
+/*
+ * Returns opaque or not.
+ * @example
+ *     OSX::NSDecimal.opaque?
+ *     => true
+ *     OSX::NSRect.opaque?
+ *     => false
+ */
 static VALUE
 rb_bs_boxed_is_opaque (VALUE rcv)
 {
@@ -474,7 +509,7 @@ rb_bs_struct_get_bs_struct (VALUE rcv)
 
   bs_struct = find_bs_boxed_for_klass(rcv);
   if (bs_struct == NULL) 
-    rb_bug("Can't get bridge support structure for the given klass %p", rcv);
+    rb_bug("Can't get bridge support structure for the given klass %p", (void *)rcv);
   if (bs_struct->type != bsBoxedStructType)
     rb_bug("Invalid bridge support boxed structure type %d", bs_struct->type);
 
@@ -515,7 +550,7 @@ rb_bs_struct_new (int argc, VALUE *argv, VALUE rcv)
       field_octype = bs_struct->opt.s.fields[i].encoding;
 
       if (!rbobj_to_ocdata(argv[i], field_octype, data + pos, NO))
-        rb_raise(rb_eArgError, "Cannot convert arg #%d of type %d to Objective-C", i, field_octype);
+        rb_raise(rb_eArgError, "Cannot convert arg #%d of type %s to Objective-C", i, field_octype);
 
       pos += ocdata_size(field_octype);
     }
@@ -693,7 +728,7 @@ rb_bs_struct_get_field_data(VALUE rcv, char **field_encoding_out)
 
   Data_Get_Struct(rcv, void, struct_data);
   if (struct_data == NULL)
-    rb_raise(rb_eRuntimeError, "Given structure %p has null data", rcv);
+    rb_raise(rb_eRuntimeError, "Given structure %p has null data", (void *)rcv);
 
   for (i = 0, data = NULL, offset = 0; 
        i < bs_struct->opt.s.field_count; 
@@ -774,7 +809,7 @@ rb_bs_struct_set (VALUE rcv, VALUE val)
 
   data = rb_bs_struct_get_field_data(rcv, &octype);
   if (!rbobj_to_ocdata(val, octype, data, NO))
-    rb_raise(rb_eRuntimeError, "Can't convert Ruby object %p of type %s to Objective-C", val, octype);
+    rb_raise(rb_eRuntimeError, "Can't convert Ruby object %p of type %s to Objective-C", (void *)val, octype);
 
   rb_ivar_set(rcv, rb_bs_struct_field_ivar_id(), val);
 
@@ -943,13 +978,8 @@ bs_cf_type_create_proxy(const char *name)
   superclass = objc_getClass("NSCFType");
   if (superclass == NULL)
     rb_bug("can't locate ObjC class NSCFType");
-#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
-  klass = objc_class_alloc(name, superclass);
-  objc_addClass(klass);
-#else
   klass = objc_allocateClassPair(superclass, name, 0);
   objc_registerClassPair(klass); 
-#endif
   return klass;
 }
 
@@ -1066,9 +1096,14 @@ bridge_support_dispatcher (int argc, VALUE *argv, VALUE rcv)
   return result;
 }
 
-static struct bsRetval default_func_retval = { bsCArrayArgUndefined, -1, "v", NO };  
+static struct bsRetval default_func_retval = { bsCArrayArgUndefined, -1, "v", NO };
 
-static VALUE 
+/*
+ * Loads brdigesupport dylib. Recommends to use OSX.require_frmework in regular cases.
+ * @param Path of bridgesupport dylib.
+ * @return nil
+ */
+static VALUE
 osx_load_bridge_support_dylib (VALUE rcv, VALUE path)
 {
   const char *cpath;
@@ -1080,12 +1115,6 @@ osx_load_bridge_support_dylib (VALUE rcv, VALUE path)
   return Qnil;
 }
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
-// DO NOT SUPPORT formal protocols with bridgesupport on 10.4 or earlier.
-// there is no objc runtime-api for gathering all protocols in objective-c 1.0.
-#define reload_protocols()
-#else
-
 static void
 reload_protocols(void) 
 {
@@ -1107,11 +1136,11 @@ reload_protocols(void)
             struct bsInformalProtocolMethod *informal_method; \
             informal_method = (struct bsInformalProtocolMethod *)malloc(sizeof(struct bsInformalProtocolMethod)); \
             ASSERT_ALLOC(informal_method); \
-            informal_method->selector = (char *)methods[j].name; \
+            informal_method->selector = (char *)sel_getName(methods[j].name); \
             informal_method->is_class_method = cmethods; \
             informal_method->encoding = strdup(methods[j].types); \
             informal_method->protocol_name = strdup(protocol_getName(p)); \
-            st_insert(t, (st_data_t)methods[j].name, (st_data_t)informal_method); \
+            st_insert(t, (st_data_t)sel_getName(methods[j].name), (st_data_t)informal_method); \
         } \
 	if (methods != NULL) { \
 	    free(methods); \
@@ -1135,8 +1164,6 @@ reload_protocols(void)
     }
 } 
 
-#endif
-
 static int
 compare_bs_arg(const void *a, const void *b)
 {
@@ -1496,17 +1523,7 @@ osx_load_bridge_support_file (VALUE mOSX, VALUE path)
             bs_cf_type->bridged_class_name = bs_cf_type->name;
           }
  
-	  // void pointer (*void) should not be any specified type.
-	  // Lion's bridgesupport file contains cftype with encoding "^v",
-	  // such as CFTypeRef in CoreFoundation or ABRecordRef in AddressBook.
-	  // (1) bsCFTypes : lookup by enconding "^v" -> NG
-	  // (2) bsCFTypes2: lookup by CFTypeID `CFTypeRef' -> OK
-	  if (strcmp(typeid_encoding, "^v") == 0) {
-	    // do not insert into bsCFTypes
-	  }
-	  else {
-	    st_insert(bsCFTypes, (st_data_t)typeid_encoding, (st_data_t)bs_cf_type);
-	  }
+          st_insert(bsCFTypes, (st_data_t)typeid_encoding, (st_data_t)bs_cf_type);
           if (bs_cf_type->type_id > 0) 
             st_insert(bsCFTypes2, (st_data_t)bs_cf_type->type_id, (st_data_t)bs_cf_type);
         }
@@ -1923,6 +1940,11 @@ osx_load_bridge_support_file (VALUE mOSX, VALUE path)
 
 #else /* !HAS_LIBXML2 */
 
+/*
+ * Loads brdigesupport file. Recommends to use OSX.require_frmework in regular cases.
+ * @param Path of bridgesupport file.
+ * @return [Moudule] Module OSX
+ */
 static VALUE
 osx_load_bridge_support_file (VALUE rcv, VALUE path)
 {
@@ -1943,6 +1965,11 @@ find_magic_cookie_const_by_value(void *value)
   return bs_const;
 }
 
+/*
+ * Imports a C constant from loaded bridgesupport.
+ * @param sym Name of constant
+ * @return A new constant imported
+ */
 static VALUE
 osx_import_c_constant (VALUE self, VALUE sym)
 {
@@ -1985,7 +2012,7 @@ osx_import_c_constant (VALUE self, VALUE sym)
     if (!ocdata_to_rbobj(Qnil, bs_const->encoding, cvalue, &value, NO))
       rb_raise(ocdataconv_err_class(), "Cannot convert the Objective-C constant '%s' as '%s' to Ruby", name, bs_const->encoding);
     rb_define_const(self, name, value);
-    BS_LOG("Imported C constant `%s' with value %p", name, value);
+    BS_LOG("Imported C constant `%s' with value %p", name, (void *)value);
   }
 
   if (name != real_name)
@@ -2002,6 +2029,9 @@ find_bs_boxed_by_encoding (const char *encoding)
 {
   struct bsBoxed *bs_boxed;
 
+  if (strncmp(encoding, "^v", 2) == 0)
+    return NULL;
+
   if (!st_lookup(bsBoxed, (st_data_t)encoding, (st_data_t *)&bs_boxed))
     return NULL;
 
@@ -2013,6 +2043,14 @@ find_bs_cf_type_by_encoding(const char *encoding)
 {
   struct bsCFType *cf_type;
 
+  // 10.7 or later, bridgesupport file contains cftype with encoding "^v".
+  // void pointer (*void) should not be any specified type.
+  // such as CFTypeRef in CoreFoundation or ABRecordRef in AddressBook.
+  // (1) bsCFTypes : lookup by enconding "^v" -> NG
+  // (2) bsCFTypes2: lookup by CFTypeID `CFTypeRef' -> OK
+  if (strncmp(encoding, "^v", 2) == 0)
+    return NULL;
+
   if (!st_lookup(bsCFTypes, (st_data_t)encoding, (st_data_t *)&cf_type))
     return NULL;
 
@@ -2121,6 +2159,12 @@ find_bs_informal_protocol_method(const char *selector, BOOL is_class_method)
     ? method : NULL;
 }
 
+/*
+ * Finds type encoding of a given method from informal protocols.
+ * @param sel Name of a method
+ * @param [Boolean] is_class_method Whether the given method is a class method or not
+ * @return [Sting] Type encoding
+ */
 static VALUE
 osx_lookup_informal_protocol_method_type (VALUE rcv, VALUE sel, 
   VALUE is_class_method)
@@ -2133,6 +2177,19 @@ osx_lookup_informal_protocol_method_type (VALUE rcv, VALUE sel,
   return method == NULL ? Qnil : rb_str_new2(method->encoding);
 }
 
+/*
+ * Document-class: OSX::Boxed < Object
+ * Representaion of C-types and C-structures in Ruby world.
+ *
+ * Information of types are described in bridgesupport files
+ * located under frameworks.
+ *
+ * @example
+ *     range = OSX.NSMakeRange(1, 3)
+ *     => #<OSX::NSRange location=1, length=3>
+ *     OSX::NSRange.superclass
+ *     => OSX::Boxed
+ */
 void
 initialize_bridge_support (VALUE mOSX)
 {
