@@ -25,8 +25,11 @@
 #import "ocdata_conv.h"
 #import "OverrideMixin.h"
 #import "internal_macros.h"
-#import "objc_compat.h"
-#import "st.h"
+#ifdef HAVE_RUBY_ST_H
+#import <ruby/st.h>
+#else
+#import <st.h>
+#endif
 
 #define BRIDGE_SUPPORT_NAME "BridgeSupport"
 
@@ -142,7 +145,11 @@ static char* framework_bridge_support_path()
 
 static void load_path_unshift(char* path)
 {
+#ifdef HAVE_RUBY_RUBY_H
+#define rb_load_path (rb_gv_get("$LOAD_PATH"))
+#else
   extern VALUE rb_load_path;
+#endif
   VALUE rpath = rb_str_new2(path);
   free(path);
 
@@ -301,19 +308,25 @@ static int rubycocoa_initialized_p()
   return rubycocoa_initialized_flag;
 }
 
-// exported and used by internal_macros.h 
+// exported and used by internal_macros.h
 VALUE rubycocoa_debug = Qfalse;
+VALUE rubycocoa_use_oc2rbCache = Qtrue;
 
 static BOOL rb_cocoa_check_for_multiple_libruby(void);
 static const char *rb_cocoa_path_libruby(void);
-static void RBCocoaInstallRubyThreadSchedulerHooks(void);
+
+static void rb_cocoa_use_cache_setter(VALUE val, ID id)
+{
+  rubycocoa_use_oc2rbCache = RTEST(val);
+  if (!RTEST(val))
+    clear_oc2rb_cache();
+}
 
 static void rubycocoa_init()
 {
   if (! rubycocoa_initialized_flag) {
     // initialize the threading hooks
     rb_cocoa_check_for_multiple_libruby();
-    RBCocoaInstallRubyThreadSchedulerHooks();
 
     init_rb2oc_cache();    // initialize the Ruby->ObjC internal cache
     init_oc2rb_cache();    // initialize the ObjC->Ruby internal cache
@@ -323,9 +336,13 @@ static void rubycocoa_init()
     init_ovmix();
     load_path_unshift(framework_ruby_path()); // PATH_TO_FRAMEWORK/Resources/ruby
     sign_path_unshift(framework_bridge_support_path());
+#ifdef HAVE_RUBY_ENCODING_H
+    init_encoding_conversion(); // initialize Ruby<->ObjC string encoding conversion
+#endif
     rubycocoa_initialized_flag = 1;
     rb_define_variable("$RUBYCOCOA_DEBUG", &ruby_debug);
     rubycocoa_debug = getenv("RUBYCOCOA_DEBUG") != NULL;
+    rb_define_hooked_variable("$RUBYCOCOA_USE_OC2RBCACHE", &rubycocoa_use_oc2rbCache, 0, &rb_cocoa_use_cache_setter);
   }
 }
 
@@ -334,7 +351,11 @@ rubycocoa_bundle_init(const char* program,
                       bundle_support_program_loader_t loader,
                       Class klass, id param)
 {
+#ifdef HAVE_RUBY_RUBY_H
+  extern void Init_stack(volatile VALUE*);
+#else
   extern void Init_stack(VALUE*);
+#endif
   int state;
   Init_stack((void*)&state);
   if (! rubycocoa_initialized_p()) {
@@ -355,17 +376,26 @@ rubycocoa_app_init(const char* program,
                    bundle_support_program_loader_t loader,
                    int argc, const char* argv[], id param)
 {
-  extern void Init_stack(VALUE*);
-  int state;
   int ruby_argc;
   const char** ruby_argv;
+#ifdef HAVE_RUBY_RUBY_H
+  void* node = NULL;
+  RUBY_INIT_STACK;
+#else
+  int state;
+  extern void Init_stack(VALUE*);
   Init_stack((void*)&state);
+#endif
   if (! rubycocoa_initialized_p()) {
     ruby_init();
     ruby_init_loadpath();
     rubycocoa_init();
     ruby_argc = prepare_argv(argc, argv, program, &ruby_argv);
+#ifdef HAVE_RUBY_RUBY_H
+    node = ruby_options(ruby_argc, (char**) ruby_argv);
+#else
     ruby_options(ruby_argc, (char**) ruby_argv);
+#endif
     rubycocoa_set_frequently_init_stack(0);
   }
   load_path_unshift(resource_path());
@@ -374,7 +404,11 @@ rubycocoa_app_init(const char* program,
   framework_paths_unshift(shared_frameworks_path());
   // call ruby_run() launched from standaloneify.rb
   if (getenv("RUBYCOCOA_STANDALONEIFYING?")) {
+#ifdef HAVE_RUBY_RUBY_H
+    exit(ruby_run_node(node));
+#else
     ruby_run();
+#endif
   }
   return loader(program, nil, param);
 }
@@ -444,17 +478,28 @@ RBApplicationMain(const char* rb_program_path, int argc, const char* argv[])
 {
   int ruby_argc;
   const char** ruby_argv;
+#ifdef HAVE_RUBY_RUBY_H
+  void* node;
+#endif
 
   if (! rubycocoa_initialized_p()) {
     ruby_init();
     ruby_argc = prepare_argv(argc, argv, rb_program_path, &ruby_argv);
+#ifdef HAVE_RUBY_RUBY_H
+    node = ruby_options(ruby_argc, (char**) ruby_argv);
+#else
     ruby_options(ruby_argc, (char**) ruby_argv);
+#endif
     rubycocoa_init();
     load_path_unshift(resource_path()); // PATH_TO_BUNDLE/Contents/resources
     sign_path_unshift(bridge_support_path());
     framework_paths_unshift(private_frameworks_path());
     framework_paths_unshift(shared_frameworks_path());
+#ifdef HAVE_RUBY_RUBY_H
+    ruby_run_node(node);
+#else
     ruby_run();
+#endif
   }
   return 0;
 }
@@ -548,30 +593,6 @@ static NSThread *rb_cocoa_main_nsthread;
    rb_cocoa_thread_context */
 static struct st_table *rb_cocoa_thread_state;
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
-/* <= TIGER */
-
-/* Offsets into NSThread object of important instance vars */
-static int rb_cocoa_NSThread_autoreleasePool;
-static int rb_cocoa_NSThread_excHandlers;
-
-/* Access to the autoreleasePool ivar of NSThread */
-# define NSTHREAD_autoreleasePool_get(t) \
-    (*(void**)( ((char*)t) + rb_cocoa_NSThread_autoreleasePool ))
-# define NSTHREAD_autoreleasePool_set(t, d) \
-    (NSTHREAD_autoreleasePool_get(t) = d)
-# define NSTHREAD_autoreleasePool_init(t) (NULL)
-
-/* Access to the excHandlers ivar of NSThread */
-# define NSTHREAD_excHandlers_get(t) \
-    (*(void**)( ((char*)t) + rb_cocoa_NSThread_excHandlers ))
-# define NSTHREAD_excHandlers_set(t, d) \
-    (NSTHREAD_excHandlers_get(t) = d)
-# define NSTHREAD_excHandlers_free(t, d)  
-# define NSTHREAD_excHandlers_init(t) (NULL)
-
-#else /* > TIGER */
-
 #include <setjmp.h>
 
 typedef struct {
@@ -681,8 +702,6 @@ static void *NSTHREAD_excHandlers_init(void *t)
 
 # define NSTHREAD_NEEDS_TO_SAVE 0 
 
-#endif
-
 struct rb_cocoa_thread_context
 {
   void *  excHandlers;
@@ -928,95 +947,6 @@ static void rb_cocoa_thread_schedule_hook(rb_threadswitch_event_t event,
   }
 }
 
-static void RBCocoaInstallRubyThreadSchedulerHooks()
-{
-  SInt32 version;
-  if (Gestalt(gestaltSystemVersion, &version) == noErr) {
-    if (version >= 0x1060) {
-      /* The threading support is not implemented yet in 10.6. */
-      return;
-    }
-  }
-
-  if (getenv("RUBYCOCOA_THREAD_HOOK_DISABLE") != NULL) {
-    if (rb_cocoa_thread_debug) {
-      NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: warning: disabled hooks due to RUBYCOCOA_THREAD_HOOK_DISABLE environment variable");
-    }
-    return;
-  }
-
-  rb_cocoa_thread_debug = getenv("RUBYCOCOA_THREAD_DEBUG") != NULL;
-  
-  // TODO dyld "NS" APIs are deprecated. the following code should be rewritten
-  // with dlopen/dlsym. see mach-o/dyld.h.
-  struct mach_header *libruby_image = 
-    NSAddImage(rb_cocoa_path_libruby(), NSADDIMAGE_OPTION_RETURN_ON_ERROR);
-  NSSymbol threadswitch_symbol = 
-    NSLookupSymbolInImage(libruby_image, "_rb_add_threadswitch_hook", NSLOOKUPSYMBOLINIMAGE_OPTION_RETURN_ON_ERROR); 
-  NSSymbol ruby_init_symbol = 
-    NSLookupSymbolInImage(libruby_image, "_ruby_init", NSLOOKUPSYMBOLINIMAGE_OPTION_RETURN_ON_ERROR); 
-  
-  if (threadswitch_symbol == NULL) {
-    if (rb_cocoa_thread_debug) {
-      NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: warning: rb_set_cocoa_thread_hooks not present in ruby core");
-    }
-    return;
-  }
-  
-  if (NSModuleForSymbol(threadswitch_symbol) 
-      != NSModuleForSymbol(ruby_init_symbol)) {
-    NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: warning: rb_set_cocoa_thread_hooks is linked from a different library (%s) than ruby_init (%s)",
-      NSNameOfModule(NSModuleForSymbol(threadswitch_symbol)), NSNameOfModule(NSModuleForSymbol(ruby_init_symbol)));
-    return;
-  }
-
-  rb_cocoa_thread_state = st_init_numtable();  
-  rb_cocoa_main_nsthread = [NSThread currentThread];
-  
-  rb_cocoa_NSThread_class = objc_lookUpClass("NSThread");
-  if (rb_cocoa_NSThread_class == NULL) {
-    NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: couldn't find NSThread class");
-    return;
-  }
-
-#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4  
-  Ivar v;
-
-  v = class_getInstanceVariable(rb_cocoa_NSThread_class, "autoreleasePool");
-  if (v == NULL) {
-    NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: couldn't find autoreleasePool ivar");
-    return;
-  }
-  
-  rb_cocoa_NSThread_autoreleasePool = v->ivar_offset;
-  
-  v = class_getInstanceVariable(rb_cocoa_NSThread_class, "excHandlers");
-  if (v == NULL) {
-    NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: couldn't find excHandlers ivar");
-    return;
-  }
-  
-  rb_cocoa_NSThread_excHandlers = v->ivar_offset;
-#endif
-
-  rb_cocoa_currentThread_SEL = @selector(currentThread);
-  Method method = class_getClassMethod(rb_cocoa_NSThread_class, 
-    rb_cocoa_currentThread_SEL);
-  if (method == NULL) {
-    NSLog(@"RBCocoaInstallRubyThreadSchedulerHooks: can't find IMP for +[NSThread currentThread]");
-    return;
-  }
-  rb_cocoa_NSThread_currentThread_imp = (NSThread*(*)(id, SEL)) 
-    method_getImplementation(method);
-
-  /* Finally, register the hook with the ruby core */
-  rb_add_threadswitch_hook(rb_cocoa_thread_schedule_hook);
-  rb_cocoa_did_install_thread_hooks = YES;
-
-  DLOG("Thread hooks done, main Ruby thread is %p\n", 
-    (void *)rb_thread_current());
-}
-
 @interface RBRuntime : NSObject
 @end
 
