@@ -7,7 +7,7 @@
  * LGPL. See the COPYRIGHT file for more information.
  */
 
-#import "osx_ruby.h"
+#import <ruby.h>
 #import <objc/objc.h>
 
 struct _objcid_data {
@@ -29,4 +29,4 @@ VALUE objcid_new_with_ocid(VALUE klass, id ocid);
 #define OBJCID_IDPTR(rcv)  (&OBJCID_ID(rcv))
 
 /** initial loading **/
-VALUE init_cls_ObjcID (VALUE outer);
+void init_cls_ObjcID (VALUE mOSX);
