@@ -7,7 +7,7 @@
  * LGPL. See the COPYRIGHT file for more information.
  */
 
-#import "osx_ruby.h"
+#import <ruby.h>
 
 /** class methods **/
 VALUE objcptr_s_class ();
@@ -18,4 +18,4 @@ void* objcptr_cptr (VALUE rcv);
 long objcptr_allocated_size(VALUE rcv);
 
 /** initial loading **/
-VALUE init_cls_ObjcPtr (VALUE outer);
+void init_cls_ObjcPtr (VALUE mOSX);
