@@ -15,7 +15,11 @@
 #import "cls_objcid.h"
 #import "cls_objcptr.h"
 #import "internal_macros.h"
+#ifdef HAVE_RUBY_ST_H
+#import <ruby/st.h>
+#else
 #import <st.h>
+#endif
 #include <sys/mman.h>   // for mmap()
 
 #define FFI_LOG(fmt, args...) DLOG("LIBFFI", fmt, ##args)
@@ -98,7 +102,7 @@ fake_ary_ffi_type_cary (unsigned bytes, unsigned align, const char *octypestr)
 
   assert(bytes > 0);
 
-  if (sscanf(octypestr, "[%d%1s]", &count, octype) == 0 || count == 0) {
+  if (sscanf(octypestr, "[%d%1s]", &count, (char *)octype) == 0 || count == 0) {
     FFI_LOG("scan failed for octype '%s'", octypestr);
     return fake_ary_ffi_type ( bytes, align);
   }
@@ -449,7 +453,7 @@ rb_ffi_dispatch (
           prev_len = arg_values[bs_arg->c_ary_type_value + argc_delta];
           if (prev_len != NULL && (*prev_len > len || *prev_len < 0))
             return rb_err_new(rb_eArgError, "Incorrect array length of argument #%d (expected a non negative value greater or equal to %d, got %d)", i, len, *prev_len);
-          FFI_LOG("arg[%d] (%p) : %s (defined as a C array delimited by arg #%d in the metadata)", i, arg, octype_str, bs_arg->c_ary_type_value);
+          FFI_LOG("arg[%d] (%p) : %s (defined as a C array delimited by arg #%d in the metadata)", i, (void *)arg, octype_str, bs_arg->c_ary_type_value);
         }
         value = OCDATA_ALLOCA(octype_str);
         if (len > 0)
@@ -457,7 +461,7 @@ rb_ffi_dispatch (
       }
       // Regular argument. 
       else {
-        FFI_LOG("arg[%d] (%p) : %s", i, arg, octype_str);
+        FFI_LOG("arg[%d] (%p) : %s", i, (void *)arg, octype_str);
         len = 0;
         value = OCDATA_ALLOCA(octype_str);
       }
@@ -630,6 +634,7 @@ rb_ffi_dispatch (
                     octype_str = fake_octype_str;
                   }
                   else {
+                    // ASCII-8BIT on ruby-2.0 or later
                     rbval = rb_str_new((char *)value, 
                       length_value * ocdata_size(octype_str));
                   }
@@ -641,10 +646,12 @@ rb_ffi_dispatch (
               break;
 
             case bsCArrayArgFixedLength:
+              // ASCII-8BIT on ruby-2.0 or later
               rbval = rb_str_new((char *)value, bs_arg->c_ary_type_value);
               break;
 
             case bsCArrayArgDelimitedByNull:
+              // ASCII-8BIT on ruby-2.0 or later
               rbval = rb_str_new2((char *)value);
               break;
 
