@@ -7,7 +7,7 @@
  * LGPL. See the COPYRIGHT file for more information.
  */
 
-#import "osx_ruby.h"
+#import <ruby.h>
 #import "ocdata_conv.h"
 #import "mdl_osxobjc.h"
 #import <Foundation/Foundation.h>
@@ -18,7 +18,6 @@
 #import "BridgeSupport.h"
 #import "internal_macros.h"
 #import "ocexception.h"
-#import "objc_compat.h"
 #import <sys/mman.h>
 
 #if __LP64__
@@ -61,7 +60,7 @@ ocm_retain_arg_if_necessary (VALUE result, BOOL is_result, void *context)
 
       if (!is_result
           || NIL_P(rcv)
-          || strncmp((const char *)selector, "init", 4) != 0
+          || strncmp(sel_getName(selector), "init", 4) != 0
           || OBJCID_ID(rcv) == OBJCID_ID(result)
           || !OBJCID_DATA_PTR(rcv)->retained) { 
 
@@ -322,7 +321,7 @@ ocm_send(int argc, VALUE* argv, VALUE rcv, VALUE* result)
     // as the target class may override the invocation dispatching methods (as NSUndoManager).
     methodSignature = [oc_rcv methodSignatureForSelector:selector];
     if (methodSignature == nil) {
-      exception = rb_err_new(ocmsgsend_err_class(), "Can't get Objective-C method signature for selector '%s' of receiver %s", (char *) selector, RSTRING_PTR(rb_inspect(rcv)));
+      exception = rb_err_new(rb_eNoMethodError, "Can't get Objective-C method signature for selector '%s' of receiver %s", sel_getName(selector), RSTRING_PTR(rb_inspect(rcv)));
       goto bails;
     }
     // Let's use the regular message dispatcher.
@@ -361,7 +360,7 @@ ocm_send(int argc, VALUE* argv, VALUE rcv, VALUE* result)
   argc--; // skip is predicate flag
   argv++;
 
-  OBJWRP_LOG("ocm_send (%s%c%s): args_count=%d ret_type=%s", class_getName(klass), is_class_method ? '.' : '#', selector, argc, methodReturnType);
+  OBJWRP_LOG("ocm_send (%s%c%s): args_count=%d ret_type=%s", class_getName(klass), is_class_method ? '.' : '#', sel_getName(selector), argc, methodReturnType);
 
   // Easy case: a method returning ID (or nothing) _and_ without argument.
   // We don't need libffi here, we can just call it (faster).
@@ -372,7 +371,7 @@ ocm_send(int argc, VALUE* argv, VALUE rcv, VALUE* result)
 
     exception = Qnil;
     @try {
-      OBJWRP_LOG("direct call easy method %s imp %p", (const char *)selector, imp);
+      OBJWRP_LOG("direct call easy method %s imp %p", sel_getName(selector), imp);
       val = (*imp)(oc_rcv, selector);
     }
     @catch (id oc_exception) {
@@ -408,7 +407,7 @@ ocm_send(int argc, VALUE* argv, VALUE rcv, VALUE* result)
 
   expected_argc = numberOfArguments;
 
-  bs_method = find_bs_method(klass, (const char *) selector, is_class_method); 
+  bs_method = find_bs_method(klass, sel_getName(selector), is_class_method); 
   if (bs_method != NULL) {
     OBJWRP_LOG("found metadata description\n");
     if (bs_method->ignore) {
@@ -472,7 +471,7 @@ ocm_send(int argc, VALUE* argv, VALUE rcv, VALUE* result)
     result);
 
 success:
-  OBJWRP_LOG("ocm_send (%s) done%s", (const char *)selector, NIL_P(exception) ? "" : " with exception");
+  OBJWRP_LOG("ocm_send (%s) done%s", sel_getName(selector), NIL_P(exception) ? "" : " with exception");
 
 bails:
   if (methodReturnType != NULL)
@@ -491,8 +490,13 @@ bails:
 
 /*************************************************/
 
+
+/*
+ * Tests whether the reciever responds to a given Objective-C selector.
+ * @param [String] sel ruby-style method name, like "foo_bar_".
+ */
 static VALUE
-wrapper_ocm_responds_p(VALUE rcv, VALUE sel)
+wrapper_ocm_respond_to_p(VALUE rcv, VALUE sel)
 {
   id oc_rcv;
   SEL oc_sel = rbobj_to_nssel(sel);
@@ -513,6 +517,9 @@ wrapper_ocm_conforms_p(VALUE rcv, VALUE name)
 }
 #endif
 
+/*
+ * Sends Objetive-C message to the reciever.
+ */
 static VALUE
 wrapper_ocm_send(int argc, VALUE* argv, VALUE rcv)
 {
@@ -527,6 +534,9 @@ wrapper_ocm_send(int argc, VALUE* argv, VALUE rcv)
   return result;
 }
 
+/*
+ * @return [NSString]
+ */
 static VALUE
 wrapper_to_s (VALUE rcv)
 {
@@ -546,32 +556,23 @@ static void
 _ary_push_objc_methods (VALUE ary, Class cls, int recur)
 {
   Class superclass = class_getSuperclass(cls);
-#if MAC_OS_X_VERSION_MAX_ALLOWED > MAC_OS_X_VERSION_10_4
   Method *methods;
   unsigned int i, count;
   methods = class_copyMethodList(cls, &count);
   for (i = 0; i < count; i++)
-    rb_ary_push(ary, rb_str_new2((const char *)method_getName(methods[i])));
+    rb_ary_push(ary, rb_str_new2(sel_getName(method_getName(methods[i]))));
   free(methods);
-#else
-  void* iterator = NULL;
-  struct objc_method_list* list;
-
-  while (list = class_nextMethodList(cls, &iterator)) {
-    int i;
-    struct objc_method* methods = list->method_list;
-    
-    for (i = 0; i < list->method_count; i++) {
-      rb_ary_push (ary, rb_str_new2((const char*)(methods[i].method_name)));
-    }
-  }
-#endif
 
   if (recur && superclass != NULL && !class_isMetaClass(cls))
     _ary_push_objc_methods (ary, superclass, recur);
   rb_funcall(ary, rb_intern("uniq!"), 0);
 }
 
+/*
+ * Returns an array of Objective-C methods of the reciever.
+ * Like Object#methods.
+ * @return [Array]
+ */
 static VALUE
 wrapper_objc_methods (VALUE rcv)
 {
@@ -580,10 +581,15 @@ wrapper_objc_methods (VALUE rcv)
 
   ary = rb_ary_new();
   rbobj_to_nsobj(rcv, &oc_rcv);
-  _ary_push_objc_methods (ary, oc_rcv->isa, 1);
+  _ary_push_objc_methods (ary, object_getClass(oc_rcv), 1);
   return ary;
 }
 
+/*
+ * Returns an array of Objective-C instance methods of the reciever class.
+ * Like Module#instance_methods.
+ * @return [Array]
+ */
 static VALUE
 wrapper_objc_instance_methods (int argc, VALUE* argv, VALUE rcv)
 {
@@ -598,6 +604,11 @@ wrapper_objc_instance_methods (int argc, VALUE* argv, VALUE rcv)
   return ary;
 }
 
+/*
+ * Returns an array of Objective-C class methods of the reciever class.
+ * Like Object#sigleton_methods.
+ * @return [Array]
+ */
 static VALUE
 wrapper_objc_class_methods (int argc, VALUE* argv, VALUE rcv)
 {
@@ -608,7 +619,7 @@ wrapper_objc_class_methods (int argc, VALUE* argv, VALUE rcv)
   recur = (argc == 0) ? 1 : RTEST(argv[0]);
   ary = rb_ary_new();
   rbobj_to_nsobj(rcv, &oc_rcv);
-  _ary_push_objc_methods (ary, oc_rcv->isa, recur);
+  _ary_push_objc_methods (ary, object_getClass(oc_rcv), recur);
   return ary;
 }
 
@@ -636,6 +647,17 @@ _name_to_selstr (VALUE name)
   return name;
 }
 
+/*
+ * Returns Objective-C type encodings of a given selector.
+ * @param [String] name
+ * @return [String]
+ *
+ * @example
+ *     str = OSX::NSString.stringWithString('RubyCocoa')
+ *     str.objc_method_type('compare:options:') => "q32@0:8@16Q24"
+ *     str.objc_method_type('compare_options_') => "q32@0:8@16Q24"
+ *     str.objc_method_type('hasPrefix:') => "c24@0:8@16"
+ */
 static VALUE
 wrapper_objc_method_type (VALUE rcv, VALUE name)
 {
@@ -644,11 +666,17 @@ wrapper_objc_method_type (VALUE rcv, VALUE name)
 
   rbobj_to_nsobj(rcv, &oc_rcv);
   name = _name_to_selstr (name);
-  str = _objc_method_type (oc_rcv->isa, StringValuePtr(name));
+  str = _objc_method_type (object_getClass(oc_rcv), StringValuePtr(name));
   if (str == NULL) return Qnil;
   return rb_str_new2(str);
 }
 
+/*
+ * Returns Objective-C type encodings of a given selector.
+ * @param [String] name
+ * @return [String]
+ * @see OSX::OCObjWrapper#objc_method_type
+ */
 static VALUE
 wrapper_objc_instance_method_type (VALUE rcv, VALUE name)
 {
@@ -662,6 +690,12 @@ wrapper_objc_instance_method_type (VALUE rcv, VALUE name)
   return rb_str_new2(str);
 }
 
+/*
+ * Returns Objective-C type encodings of a given selector.
+ * @param [String] name
+ * @return [String]
+ * @see OSX::OCObjWrapper#objc_method_type
+ */
 static VALUE
 wrapper_objc_class_method_type (VALUE rcv, VALUE name)
 {
@@ -670,7 +704,7 @@ wrapper_objc_class_method_type (VALUE rcv, VALUE name)
 
   rbobj_to_nsobj(rcv, &oc_rcv);
   name = _name_to_selstr (name);
-  str = _objc_method_type (oc_rcv->isa, StringValuePtr(name));
+  str = _objc_method_type (object_getClass(oc_rcv), StringValuePtr(name));
   if (str == NULL) return Qnil;
   return rb_str_new2(str);
 }
@@ -689,13 +723,19 @@ _objc_alias_method (Class klass, VALUE new, VALUE old)
 
   // warn if trying to alias a method that isn't a member of the specified class
   if (me == NULL)
-    rb_raise(rb_eRuntimeError, "could not alias '%s' for '%s' to class '%s': Objective-C cannot find it in the class", (char *)new_name, (char *)old_name, class_getName(klass));
+    rb_raise(rb_eRuntimeError, "could not alias '%s' for '%s' to class '%s': Objective-C cannot find it in the class", sel_getName(new_name), sel_getName(old_name), class_getName(klass));
   
   class_addMethod(klass, new_name, method_getImplementation(me), method_getTypeEncoding(me));
   
   return nil;
 }
 
+/*
+ * Aliases Objective-C method to given new name.
+ * @param [String] new
+ * @param [String] old
+ * @return [self]
+ */
 static VALUE
 wrapper_objc_alias_method (VALUE rcv, VALUE new, VALUE old)
 {
@@ -705,31 +745,46 @@ wrapper_objc_alias_method (VALUE rcv, VALUE new, VALUE old)
   return rcv;
 }
 
+/*
+ * Aliases Objective-C method to given new name.
+ * @param [String] new
+ * @param [String] old
+ * @return [self]
+ */
 static VALUE
 wrapper_objc_alias_class_method (VALUE rcv, VALUE new, VALUE old)
 {
   id oc_rcv;
   rbobj_to_nsobj(rcv, &oc_rcv);
-  _objc_alias_method((Class)(oc_rcv->isa), new, old);
+  _objc_alias_method(object_getClass(oc_rcv), new, old);
   return rcv;
 }
 
 /*****************************************/
 
-VALUE
-init_mdl_OCObjWrapper(VALUE outer)
+/*
+ * Document-module: OSX::OCObjWrapper
+ * Document-module: OSX::OCClsWrapper
+ *
+ * Utilities for communicating Objective-C classes.
+ *
+ * Objective-C classes in Ruby world includes {OSX::OCObjWrapper}
+ * and extends {OSX::OCClsWrapper}.
+ */
+void
+init_mdl_OCObjWrapper(VALUE mOSX)
 {
-  _mObjWrapper = rb_define_module_under(outer, "OCObjWrapper");
+  _mObjWrapper = rb_define_module_under(mOSX, "OCObjWrapper");
 
-  rb_define_method(_mObjWrapper, "ocm_responds?", wrapper_ocm_responds_p, 1);
-	//rb_define_method(_mObjWrapper, "ocm_conforms?", wrapper_ocm_conforms_p, 1);
+  rb_define_method(_mObjWrapper, "ocm_respond_to?", wrapper_ocm_respond_to_p, 1);
+  rb_define_alias(_mObjWrapper, "ocm_responds?", "ocm_respond_to?");
   rb_define_method(_mObjWrapper, "ocm_send", wrapper_ocm_send, -1);
   rb_define_method(_mObjWrapper, "to_s", wrapper_to_s, 0);
 
   rb_define_method(_mObjWrapper, "objc_methods", wrapper_objc_methods, 0);
   rb_define_method(_mObjWrapper, "objc_method_type", wrapper_objc_method_type, 1);
 
-  _mClsWrapper = rb_define_module_under(outer, "OCClsWrapper");
+  _mClsWrapper = rb_define_module_under(mOSX, "OCClsWrapper");
   rb_define_method(_mClsWrapper, "objc_instance_methods", wrapper_objc_instance_methods, -1);
   rb_define_method(_mClsWrapper, "objc_class_methods", wrapper_objc_class_methods, -1);
   rb_define_method(_mClsWrapper, "objc_instance_method_type", wrapper_objc_instance_method_type, 1);
@@ -738,8 +793,6 @@ init_mdl_OCObjWrapper(VALUE outer)
   rb_define_method(_mClsWrapper, "_objc_alias_method", wrapper_objc_alias_method, 2);
   rb_define_method(_mClsWrapper, "_objc_alias_class_method", wrapper_objc_alias_class_method, 2);
 
-  rb_define_module_function(outer, "_ignore_ns_override", wrapper_ignore_ns_override, 0);
-  rb_define_module_function(outer, "_ignore_ns_override=", wrapper_ignore_ns_override_set, 1);
-
-  return Qnil;
+  rb_define_module_function(mOSX, "_ignore_ns_override", wrapper_ignore_ns_override, 0);
+  rb_define_module_function(mOSX, "_ignore_ns_override=", wrapper_ignore_ns_override_set, 1);
 }
