@@ -13,7 +13,14 @@
 #import "RBObject.h"
 #import "mdl_osxobjc.h"
 #import <CoreFoundation/CFString.h> // CFStringEncoding
-#import "st.h"
+#ifdef HAVE_RUBY_ST_H
+#import <ruby/st.h>
+#else
+#import <st.h>
+#endif
+#ifdef HAVE_RUBY_ENCODING_H
+#import <ruby/encoding.h>
+#endif
 #import "BridgeSupport.h"
 #import "internal_macros.h"
 
@@ -24,6 +31,17 @@
 static struct st_table *rb2ocCache;
 static struct st_table *oc2rbCache;
 
+/* dummy ruby string encoding for unknown Ruby<->NSString conversion */
+#ifdef HAVE_RUBY_ENCODING_H
+static int ENCINDEX_RUBYCOCOA_UNKNOWN;
+
+static struct st_table *rb2ocEncConv;
+static struct st_table *oc2rbEncConv;
+
+static CFStringEncoding strenc_rb2oc(rb_encoding *rbenc);
+static rb_encoding *strenc_oc2rb(CFStringEncoding cfenc);
+#endif
+
 static VALUE _ocid_to_rbobj (VALUE context_obj, id ocid, BOOL is_class);
 
 #if CACHE_LOCKING
@@ -36,13 +54,6 @@ static pthread_mutex_t oc2rbCacheLock;
 # define CACHE_UNLOCK(x)
 #endif
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4
-// On MacOS X 10.4 or earlier, +signatureWithObjCTypes: is a SPI 
-@interface NSMethodSignature (WarningKiller)
-+ (id) signatureWithObjCTypes:(const char*)types;
-@end
-#endif
-
 @interface RBObject (Private)
 - (id)_initWithRubyObject: (VALUE)rbobj retains: (BOOL) flag;
 @end
@@ -63,6 +74,20 @@ void init_oc2rb_cache(void)
 #endif
 }
 
+static int
+delete_cache_item(st_data_t key, VALUE value, VALUE never)
+{
+  return ST_DELETE;
+}
+
+void clear_oc2rb_cache(void)
+{
+  CACHE_LOCK(&oc2rbCacheLock);
+  // delete all cached items
+  st_foreach(oc2rbCache, delete_cache_item, Qundef);
+  CACHE_UNLOCK(&oc2rbCacheLock);
+}
+
 void remove_from_oc2rb_cache(id ocid)
 {
   CACHE_LOCK(&oc2rbCacheLock);
@@ -126,7 +151,7 @@ convert_cary(VALUE *result, void *ocdata, char *octype_str, BOOL to_ruby)
     Check_Type(ary, T_ARRAY);
     if (RARRAY_LEN(ary) > count)
       rb_raise(rb_eArgError, 
-        "Given Array expected with maximum %d elements, but got %d",
+        "Given Array expected with maximum %ld elements, but got %ld",
         count, RARRAY_LEN(ary));
 
     for (i = 0; i < RARRAY_LEN(ary); i++) {
@@ -420,6 +445,7 @@ ocdata_to_rbobj (VALUE context_obj, const char *octype_str, const void *ocdata,
       if (*(void **)ocdata == NULL)
         rbval = Qnil;
       else
+        // ASCII-8BIT on ruby-2.0 or later
         rbval = rb_str_new2(*(char **)ocdata); 
       break;
   
@@ -720,7 +746,7 @@ _ocid_to_rbobj (VALUE context_obj, id ocid, BOOL is_class)
           ? context_obj : ocobj_s_new(ocid);
     }
 
-    if (shouldCache) {
+    if (shouldCache && RUBYCOCOA_USE_OC2RBCACHE_P) {
       CACHE_LOCK(&oc2rbCacheLock);
       // Check out that the hash is still empty for us, to avoid a race 
       // condition.
@@ -795,12 +821,12 @@ funcptr_closure_handler (ffi_cif *cif, void *resp, void **args, void *userdata)
     if (!ocdata_to_rbobj(Qnil, context->argtypes[i], args[i], &arg, NO))
       rb_raise(rb_eRuntimeError, "Can't convert Objective-C argument #%d of octype '%s' to Ruby value", i, context->argtypes[i]);
 
-    DATACONV_LOG("converted arg #%d of type %s to Ruby value %p", i, context->argtypes[i], arg);
+    DATACONV_LOG("converted arg #%d of type %s to Ruby value %p", i, context->argtypes[i], (void *)arg);
 
     rb_ary_store(rb_args, i, arg);
   }
 
-  DATACONV_LOG("calling Ruby block with %d args...", RARRAY_LEN(rb_args));
+  DATACONV_LOG("calling Ruby block with %ld args...", RARRAY_LEN(rb_args));
   retval = rb_funcall2(context->block, rb_intern("call"), RARRAY_LEN(rb_args), RARRAY_PTR(rb_args));
   DATACONV_LOG("called Ruby block");
 
@@ -1073,11 +1099,11 @@ rbobj_to_ocdata (VALUE obj, const char *octype_str, void* ocdata, BOOL to_libffi
 #endif
 
     case _C_FLT:
-      *(float*)ocdata = (float) RFLOAT(rb_Float(obj))->value;
+      *(float*)ocdata = (float) RFLOAT_VALUE(rb_Float(obj));
       break;
 
     case _C_DBL:
-      *(double*)ocdata = RFLOAT(rb_Float(obj))->value;
+      *(double*)ocdata = RFLOAT_VALUE(rb_Float(obj));
       break;
 
     case _C_CHARPTR:
@@ -1110,6 +1136,7 @@ rbobj_to_ocdata (VALUE obj, const char *octype_str, void* ocdata, BOOL to_libffi
   return f_success;
 }
 
+#ifndef HAVE_RUBY_ENCODING_H
 static 
 NSStringEncoding kcode_to_nsencoding (const char* kcode) 
 { 
@@ -1123,21 +1150,63 @@ NSStringEncoding kcode_to_nsencoding (const char* kcode)
     return NSUTF8StringEncoding;
 }
 #define KCODE_NSSTRENCODING kcode_to_nsencoding(rb_get_kcode()) 
+#endif
 
 id
 rbstr_to_ocstr(VALUE obj)
 {
-  return [[[NSMutableString alloc] initWithData:[NSData dataWithBytes:RSTRING_PTR(obj)
-			    			 length: RSTRING_LEN(obj)]
-			    encoding:KCODE_NSSTRENCODING] autorelease];
+  NSStringEncoding nsenc;
+  VALUE str = obj;
+#ifdef HAVE_RUBY_ENCODING_H
+  rb_encoding *rbenc = rb_enc_get(obj);
+  CFStringEncoding cfenc;
+  cfenc = strenc_rb2oc(rbenc);
+  if (cfenc == kCFStringEncodingInvalidId && rb_enc_to_index(rbenc) != ENCINDEX_RUBYCOCOA_UNKNOWN) {
+    // try to convert into UTF-8
+    str = rb_str_conv_enc(obj, rbenc, rb_utf8_encoding());
+    if (rb_enc_get_index(str) == rb_utf8_encindex()) {
+      cfenc = kCFStringEncodingUTF8;
+    } else {
+      cfenc = kCFStringEncodingInvalidId;
+    }
+  }
+  nsenc = CFStringConvertEncodingToNSStringEncoding(cfenc);
+#else
+  nsenc = KCODE_NSSTRENCODING;
+#endif
+  return [[[NSMutableString alloc] initWithData:[NSData dataWithBytes:RSTRING_PTR(str)
+			    			 length: RSTRING_LEN(str)]
+			    encoding:nsenc] autorelease];
 }
 
 VALUE
 ocstr_to_rbstr(id ocstr)
 {
-  NSData * data = [(NSString *)ocstr dataUsingEncoding:KCODE_NSSTRENCODING
-				     allowLossyConversion:YES];
+  NSData * data;
+#ifdef HAVE_RUBY_ENCODING_H
+  CFStringEncoding cfenc = CFStringGetFastestEncoding((CFStringRef)((NSString *)ocstr));
+  rb_encoding *rbenc;
+  BOOL allowLossy = NO;
+
+  rbenc = strenc_oc2rb(cfenc);
+  if (rb_enc_to_index(rbenc) == ENCINDEX_RUBYCOCOA_UNKNOWN && cfenc != kCFStringEncodingInvalidId) {
+    // try to convert into UTF-8
+    if ([ocstr canBeConvertedToEncoding:NSUTF8StringEncoding]) {
+      cfenc = kCFStringEncodingUTF8;
+      rbenc = rb_utf8_encoding();
+    } else {
+      cfenc = kCFStringEncodingInvalidId;
+      rbenc = rb_enc_from_index(ENCINDEX_RUBYCOCOA_UNKNOWN);
+    }
+  }
+  data = [(NSString *)ocstr dataUsingEncoding:CFStringConvertEncodingToNSStringEncoding(cfenc)
+		     allowLossyConversion:allowLossy];
+  return rb_enc_str_new([data bytes], [data length], rbenc);
+#else
+  data = [(NSString *)ocstr dataUsingEncoding:KCODE_NSSTRENCODING
+		     allowLossyConversion:YES];
   return rb_str_new ([data bytes], [data length]);
+#endif
 }
 
 static void
@@ -1424,3 +1493,115 @@ set_octypes_for_format_str (char **octypes, unsigned len, char *format_str)
   for (; j < len; j++)
     octypes[j] = "@"; // _C_ID;
 }
+
+#ifdef HAVE_RUBY_ENCODING_H
+#pragma mark - string encoding conversion
+BOOL register_strenc_rb2oc(rb_encoding *rbenc, CFStringEncoding cfenc, BOOL force);
+BOOL register_strenc_oc2rb(CFStringEncoding cfenc, rb_encoding *rbenc, BOOL force);
+BOOL register_strenc_pair(rb_encoding *rbenc, CFStringEncoding cfenc, BOOL force);
+
+void init_encoding_conversion()
+{
+  rb_encoding *rbenc;
+
+  rb2ocEncConv = st_init_numtable();
+  oc2rbEncConv = st_init_numtable();
+
+  // register basic encodings
+  if ((rbenc = rb_enc_find("US-ASCII"))) {
+    register_strenc_pair(rbenc, kCFStringEncodingASCII, NO);
+  }
+  if ((rbenc = rb_enc_find("UTF-8"))) {
+    register_strenc_pair(rbenc, kCFStringEncodingUTF8, NO);
+  }
+
+  // define dummy encoding
+  ENCINDEX_RUBYCOCOA_UNKNOWN = rb_define_dummy_encoding("RUBYCOCOA_UNKNOWN");
+  if ((rbenc = rb_enc_from_index(ENCINDEX_RUBYCOCOA_UNKNOWN))) {
+    register_strenc_pair(rbenc, kCFStringEncodingInvalidId, NO);
+  } else {
+//    TODO print error
+  }
+  
+}
+
+BOOL register_strenc_rb2oc(rb_encoding *rbenc, CFStringEncoding cfenc, BOOL force)
+{
+  BOOL ok;
+  int rbenc_idx = rb_enc_to_index(rbenc);
+  CFStringEncoding cfenc2;
+
+  if (rbenc_idx < 0) {
+    return NO;
+  }
+
+  if (st_lookup(rb2ocEncConv, (st_data_t)rbenc_idx, &cfenc2)) {
+    if (!force) {
+      return NO;
+    }
+    st_delete(rb2ocEncConv, (st_data_t)rbenc_idx, NULL);
+  }
+  ok = st_insert(rb2ocEncConv, (st_data_t)rbenc_idx, (st_data_t)cfenc);
+  return ok;
+}
+
+BOOL register_strenc_oc2rb(CFStringEncoding cfenc, rb_encoding *rbenc, BOOL force)
+{
+  BOOL ok;
+  int rbenc_idx = rb_enc_to_index(rbenc);
+  int rbenc_idx2;
+
+  if (rbenc_idx < 0) {
+    return NO;
+  }
+
+  if (st_lookup(oc2rbEncConv, (st_data_t)cfenc, &rbenc_idx2)) {
+    if (!force) {
+      return NO;
+    }
+    st_delete(oc2rbEncConv, (st_data_t)cfenc, NULL);
+  }
+  ok = st_insert(oc2rbEncConv, (st_data_t)cfenc, (st_data_t)rbenc_idx);
+  return ok;
+}
+
+BOOL register_strenc_pair(rb_encoding *rbenc, CFStringEncoding cfenc, BOOL force)
+{
+  BOOL rb2oc, oc2rb;
+
+  rb2oc = register_strenc_rb2oc(rbenc, cfenc, force);
+  oc2rb = register_strenc_oc2rb(cfenc, rbenc, force);
+  return (rb2oc && oc2rb);
+}
+
+static CFStringEncoding strenc_rb2oc(rb_encoding *rbenc)
+{
+  CFStringEncoding cfenc;
+  int rbenc_idx = rb_enc_to_index(rbenc);
+
+  if (st_lookup(rb2ocEncConv, (st_data_t)rbenc_idx, &cfenc)) {
+    return cfenc; // registered
+  }
+
+  // TODO get objc string encoding from ruby encoding
+  cfenc = kCFStringEncodingInvalidId;
+  register_strenc_rb2oc(rbenc, cfenc, NO);
+
+  return cfenc;
+}
+
+static rb_encoding *strenc_oc2rb(CFStringEncoding cfenc)
+{
+  int rbenc_idx;
+
+  if (st_lookup(oc2rbEncConv, (st_data_t)cfenc, &rbenc_idx)) {
+    return rb_enc_from_index(rbenc_idx);
+  }
+
+  // TODO get ruby string encoding from objc encoding
+  rbenc_idx = ENCINDEX_RUBYCOCOA_UNKNOWN;
+  register_strenc_oc2rb(cfenc, rb_enc_from_index(rbenc_idx), NO);
+
+  return rb_enc_from_index(rbenc_idx);
+}
+#endif
