@@ -103,8 +103,6 @@ module OSX
 
   # NSString additions
   class NSString
-    include OSX::OCObjWrapper
-
     def dup
       mutableCopy
     end
@@ -153,12 +151,14 @@ module OSX
     end
 
     # responds to Ruby String methods
-    alias_method :_rbobj_respond_to?, :respond_to?
     def respond_to?(mname, private = false)
-      String.public_method_defined?(mname) or _rbobj_respond_to?(mname, private)
+      if String.public_method_defined?(mname)
+        return true
+      else
+        super
+      end
     end
 
-    alias_method :objc_method_missing, :method_missing
     def method_missing(mname, *args, &block)
       if mname == :match || mname == :=~
         i = mname == :match ? 0 : 1
@@ -181,7 +181,7 @@ module OSX
         end
       else
         # call as objc string
-        result = objc_method_missing(mname, *args)
+        result = super
       end
       result
     end
@@ -206,6 +206,8 @@ module OSX
         when Numeric,OSX::NSNumber
           n = first.to_i
           n += count if n < 0
+	  # ruby-2.0 or later allows index == length
+	  count += 1 if RUBY_VERSION >= '2.0'
           if n < 0 || count <= n
             raise IndexError, "index #{first.to_i} out of string"
           end
@@ -559,6 +561,9 @@ module OSX
     def index(pattern, pos=0)
       case pattern
       when Numeric,OSX::NSNumber
+	if RUBY_VERSION >= '2.0'
+	  raise TypeError, "\`index': type mismatch: #{pattern.class} given (TypeError)"
+	end
         i = pattern.to_i
         if 0 <= i && i < 65536
           s = OSX::NSString.stringWithFormat("%C", i)
@@ -689,7 +694,7 @@ module OSX
       if length > 0
         characterAtIndex(0)
       else
-        0
+        raise ArgumentError, "empty string"
       end
     end
     
@@ -728,6 +733,9 @@ module OSX
     def rindex(pattern, pos=self.length)
       case pattern
       when Numeric,OSX::NSNumber
+	if RUBY_VERSION >= '2.0'
+	  raise TypeError, "\`index': type mismatch: #{pattern.class} given (TypeError)"
+	end
         i = pattern.to_i
         if 0 <= i && i < 65536
           s = OSX::NSString.stringWithFormat("%C", i)
@@ -1019,6 +1027,13 @@ module OSX
       (self..max).each {|i| yield i}
       self
     end
+
+    if RUBY_VERSION >= '2.0'
+      # @note Returns UTF-8 String, not dummy encoding.
+      def fileSystemRepresentation
+        self.oc_fileSystemRepresentation.force_encoding(Encoding::UTF8_MAC).encode!("filesystem")
+      end
+    end
     
     private
     
@@ -1059,7 +1074,11 @@ module OSX
     def _read_impl_num(slice, num, count)
       num += count if num < 0
       if 0 <= num && num < count
-	c = characterAtIndex(num)
+	if RUBY_VERSION >= '2.0'
+	  c = substringWithRange([num, 1])
+	else
+	  c = characterAtIndex(num)
+	end
 	deleteCharactersInRange(OSX::NSRange.new(num, 1)) if slice
 	c
       else
@@ -1106,8 +1125,6 @@ module OSX
 
   # NSArray additions
   class NSArray
-    include OSX::OCObjWrapper
-
     def dup
       mutableCopy
     end
@@ -1195,6 +1212,9 @@ module OSX
           if n < 0 || count < n
             raise RangeError, "#{range} out of range"
           end
+	  if RUBY_VERSION >= '2.0' && value.nil?
+            raise ArgumentError, "attempt insert nil to NSArray"
+	  end
           
           if 0 <= n && n < count
             if len > 0
@@ -1951,8 +1971,6 @@ module OSX
 
   # NSDictionary additions
   class NSDictionary
-    include OSX::OCObjWrapper
-
     def dup
       mutableCopy
     end
@@ -2287,6 +2305,7 @@ module OSX
     def to_i
       self.stringValue.to_s.to_i
     end
+    alias_method :to_int, :to_i
 
     def to_f
       self.doubleValue
