@@ -9,16 +9,19 @@ require 'osx/objc/oc_wrapper'
 
 module OSX
 
+  # framework search paths.
   FRAMEWORK_PATHS = [
     '/System/Library/Frameworks',
     '/Library/Frameworks'
   ]
 
+  # bridgesupport file search paths.
   SIGN_PATHS = [
     '/System/Library/BridgeSupport', 
     '/Library/BridgeSupport'
   ]
 
+  # additional bridgesupport file search paths by environment 'BRIDGE_SUPPORT_PATH'.
   PRE_SIGN_PATHS = 
     if path = ENV['BRIDGE_SUPPORT_PATH']
       path.split(':')
@@ -41,6 +44,12 @@ module OSX
     'ImageKit' => '/System/Library/Frameworks/Quartz.framework/Frameworks/ImageKit.framework'
   }
 
+  # @!group Frameworks and BridgeSupport files
+
+  # Returns bundle and path for given framework name.
+  # @api private
+  # @param framework [String] a name of framework, such as "WebKit".
+  # @return [Array] a pair of OSX::NSBundle and String.
   def _bundle_path_for_framework(framework)
     if framework[0] == ?/
       [OSX::NSBundle.bundleWithPath(framework), framework]
@@ -58,7 +67,9 @@ module OSX
     end
   end
   module_function :_bundle_path_for_framework
-  
+
+  # "require" for OSX .framework.
+  #
   # The OSX::require_framework method imports Mac OS X frameworks and uses the
   # BridgeSupport metadata to add Ruby entry points for the framework's Classes,
   # methods, and Constants into the OSX module.
@@ -66,22 +77,23 @@ module OSX
   # The framework parameter is a reference to the framework that should be
   # imported.  This may be a full path name to a particular framework, a shortcut,
   # or a framework name.  The shortcuts are the keys listed in the
-  # <tt>QUICK_FRAMEWORKS</tt> hash.
+  # {QUICK_FRAMEWORKS} hash.
   #
   # If a framework name (with no path) is given, then the method searches a number
   # of directories.  Those directories (in search order) are:
+  #
   #   1.  /System/Library/Frameworks
   #   2.  /Library/Frameworks
   #   3.  Any directories in the RUBYCOCOA_FRAMEWORK_PATHS array, if defined
   #   4.  ENV['HOME']/Library/Frameworks, if the HOME environment variable is defined
   #
-  # When using the search paths, the <tt>.framework</tt> file type extension should
+  # When using the search paths, the `.framework` file type extension should
   # be omitted from the framework name passed to the method.
   #
-  # If the method loads the framework successfully, it returns <tt>true</tt>.
-  # If the framework was already loaded the method returns <tt>false</tt>.
+  # If the method loads the framework successfully, it returns `true`.
+  # If the framework was already loaded the method returns `false`.
   # If the method is unable to locate, or unable to load the framework then it
-  # raises an <tt>ArgumentError</tt>.
+  # raises an `ArgumentError`.
   def require_framework(framework)
     return false if framework_loaded?(framework)
     bundle, path = _bundle_path_for_framework(framework)
@@ -90,10 +102,13 @@ module OSX
       raise ArgumentError, "Can't load framework '#{framework}'" 
     end
     load_bridge_support_signatures(path)
+    ns_import_all if RUBY_VERSION >= '2.0' # create ruby classes from Objective-C
     return true
   end
   module_function :require_framework
 
+  # Returns whether a framework is loaded of not.
+  # @param framework [String] a name of framework, such as "WebKit".
   def framework_loaded?(framework)
     bundle, path = _bundle_path_for_framework(framework)
     unless bundle.nil?
@@ -103,8 +118,8 @@ module OSX
       else
         # CoreFoundation/Foundation are linked at built-time.
         id = bundle.bundleIdentifier
-        loaded = (id.isEqualToString('com.apple.CoreFoundation') or 
-                  id.isEqualToString('com.apple.Foundation'))
+        loaded = (id.isEqualToString?('com.apple.CoreFoundation') or 
+                  id.isEqualToString?('com.apple.Foundation'))
       end
       loaded
     else
@@ -113,6 +128,7 @@ module OSX
   end
   module_function :framework_loaded?
 
+  # @api private
   def __load_bridge_support_file__(dir, framework_name)
     @bridge_support_signatures_loaded_marks ||= {}
     return true if @bridge_support_signatures_loaded_marks[framework_name]
@@ -134,6 +150,10 @@ module OSX
   end
   module_function :__load_bridge_support_file__
 
+  # Loads BridgeSupport file of given framework.
+  # @param framework [String] a name or fullpath of framework, such as "WebKit"
+  #                           or "/System/Library/Frameworks/WebKit.framework".
+  # @return [Boolean]
   def load_bridge_support_signatures(framework)
     # First, look into the pre paths.  
     fname = framework[0] == ?/ ? File.basename(framework, '.framework') : framework
@@ -181,6 +201,10 @@ module OSX
   end
   module_function :load_bridge_support_signatures
 
+  # @!endgroup
+
+  # @!group Objective-C Classes and methods
+
   # Load C constants/classes lazily.
   def self.const_missing(c)
     begin
@@ -190,6 +214,8 @@ module OSX
     end
   end
 
+  # Overrides const_missing.
+  # @api private
   def self.included(m)
     if m.respond_to? :const_missing
       m.module_eval do
@@ -213,12 +239,10 @@ module OSX
     end
   end
   
-  # Load the foundation frameworks.
-  OSX.load_bridge_support_signatures('CoreFoundation')
-  OSX.load_bridge_support_signatures('Foundation')
-
   # create Ruby's class for Cocoa class,
   # then define Constant under module 'OSX'.
+  # @param sym [Symbol, String] name of Objective-C class.
+  # @return [Class]
   def ns_import(sym)
     if not OSX.const_defined?(sym)
       NSLog("importing #{sym}...") if OSX._debug?
@@ -233,7 +257,27 @@ module OSX
   end
   module_function :ns_import
 
+  # Runs {OSX.ns_import} for all Cococa classes.
+  # RubyCocoa invokes `ns_import_all` implicitly in {OSX.require_framework}.
+  # @return [nil]
+  # @since 1.2.0
+  def ns_import_all
+    OSX.objc_classnames do |klassname|
+      # ignore private classes, such as starting with "_".
+      if /\A[A-Z]/ =~ klassname
+        if OSX.const_defined?(klassname)
+	  next
+	end
+	OSX.ns_import(klassname)
+      end
+    end
+    return nil
+  end
+  module_function :ns_import_all
+
   # create Ruby's class for Cocoa class
+  # @param occls [Objective-C Class]
+  # @return [Class]
   def class_new_for_occlass(occls)
     superclass = _objc_lookup_superclass(occls)
     klass = Class.new(superclass)
@@ -253,6 +297,8 @@ module OSX
   end
   module_function :class_new_for_occlass 
  
+  # Returns super class of given Objecitve-C class.
+  # @api private
   def _objc_lookup_superclass(occls)
     occls_superclass = occls.oc_superclass
     if occls_superclass.nil? or occls_superclass.__ocid__ == occls.__ocid__ 
@@ -275,14 +321,19 @@ module OSX
   end
   module_function :_objc_lookup_superclass
 
+  # @!endgroup
+
+  # Defines Objecitve-C objects' behaviors.
   module NSBehaviorAttachment
 
+    # An error message.
+    # @api private
     ERRMSG_FOR_RESTRICT_NEW = "use 'alloc.initXXX' to instantiate Cocoa Object"
 
     # restrict creating an instance by Class#new, unless the Objective-C class 
     # really responds to the new selector.
     def new
-      if ocm_responds?(:new)
+      if ocm_respond_to?(:new)
         objc_send(:new)
       else
         raise ERRMSG_FOR_RESTRICT_NEW
@@ -291,9 +342,11 @@ module OSX
 
     # initializer for definition of a derived class of a class on
     # Objective-C World.
+    # @api private
+    # @return [Boolean]
     def ns_inherited()
       return if ns_inherited?
-      kls_name = self.name.split('::')[-1]
+      kls_name = self.name.to_s.split('::')[-1]
       if kls_name
         spr_name = superclass.name.split('::')[-1]
         occls = OSX.objc_derived_class_new(self, kls_name, spr_name)
@@ -303,12 +356,15 @@ module OSX
       @inherited = true
     end
 
+    # Returns whether receiver class is inherited from Objecitve-C class or not.
+    # @api private
     def ns_inherited?
       return defined?(@inherited) && @inherited
     end
 
     # declare to override instance methods of super class which is
     # defined by Objective-C.
+    # @deprecated DO NOT use. RubyCocoa automatically overrides Objective-C methods.
     def ns_overrides(*args)
       warn "#{caller[0]}: ns_overrides is no longer necessary, should not be called anymore and will be removed in a next release. Please update your code to not use it."
     end
@@ -323,6 +379,7 @@ module OSX
     end
     alias_method :ib_outlet, :ib_outlets
 
+    # @deprecated use ib_outlet.
     def ns_outlets(*args)
       warn "#{caller[0]}:: ns_outlet(s) is deprecated, and will be removed in a next release. Please use ib_outlet(s) instead."
       ib_outlets(*args)
@@ -335,18 +392,13 @@ module OSX
       define_method(name, blk) if block_given?
     end
 
+    # Overrides automatically Objective-C methods.
+    # @api private
     def _ns_behavior_method_added(sym, class_method)
       return if OSX._ignore_ns_override
-      sel = sym.to_s.gsub(/([^_])_/, '\1:') 
-      arity = if defined?(@__imported_arity) and @__imported_arity != nil \
-              and RUBY_VERSION < "1.8.5"
-        # This is a workaround for a Ruby 1.8.2 issue, the real arity is 
-        # provided by _register_method. 
-        @__imported_arity
-      else
-        m = class_method ? method(sym) : instance_method(sym)
-        m.arity
-      end
+      sel = sym.to_s.gsub(/([^_])_/, '\1:')
+      m = class_method ? method(sym) : instance_method(sym)
+      arity = m.arity
       sel << ':' if arity > 0 and /[^:]\z/ =~ sel
       mtype = nil
       if _ns_enable_override?(sel, class_method) or
@@ -359,10 +411,16 @@ module OSX
       end
     end
 
+    # Returns whether override Objective-C methods with given name is enable or not.
+    # @param sel [Symbol, String]
+    # @param class_method [Boolean]
+    # @api private
     def _ns_enable_override?(sel, class_method)
       ns_inherited? and (class_method ? self.objc_method_type(sel) : self.objc_instance_method_type(sel))
     end
 
+    # Returns whether given typefmt do not contains any parameters.
+    # @api private
     def _no_param_method?(typefmt)
       if typefmt[0] == ?{
         count = 1
@@ -380,6 +438,8 @@ module OSX
       end
     end
 
+    # Adds an method to Objective-C class.
+    # @api private
     def _objc_export(name, types, class_method)
       typefmt = _types_to_typefmt(types)
       name = name.to_s
@@ -388,34 +448,35 @@ module OSX
       OSX.objc_class_method_add(self, name, class_method, typefmt)
     end
 
+    # Defines Objective-C instance method.
     def def_objc_method(name, types, &blk)
       if block_given? then
-        objc_method(name, types, &blk) 
+        objc_method(name, types, &blk)
       else
         raise ArgumentError, "block for method implementation expected"
       end
     end
 
+    # Defines Objective-C instance method.
+    # @param name
     def objc_method(name, types=['id'], &blk)
       define_method(name, blk) if block_given?
       _objc_export(name, types, false)
     end
 
+    # Defines Objective-C class method.
     def objc_class_method(name, types=['id'])
       _objc_export(name, types, true)
     end
 
-    def objc_export(name, types)
-      warn "#{caller[0]}: objc_export is deprecated, and will be removed in a next release. please use objc_method instead."
-      objc_method(name, types)
-    end
-
+    # Aliases Objective-C method.
     def objc_alias_method(new, old)
       new_sel = new.to_s.gsub(/([^_])_/, '\1:')
       old_sel = old.to_s.gsub(/([^_])_/, '\1:')
       _objc_alias_method(new, old)
     end
 
+    # Aliases Objective-C classs method.
     def objc_alias_class_method(new, old)
       new_sel = new.to_s.gsub(/([^_])_/, '\1:')
       old_sel = old.to_s.gsub(/([^_])_/, '\1:')
@@ -445,6 +506,7 @@ module OSX
       :ulonglong => 'Q',
       :cstr     => '*',
     }
+    # @api private
     def _types_to_typefmt(types)
       return types.strip if types.is_a?(String)
       raise ArgumentError, "Array or String (as type format) expected (got #{types.klass} instead)" unless types.is_a?(Array)
@@ -464,22 +526,24 @@ module OSX
 
   end       # module OSX::NSBehaviorAttachment
 
+  # Defines internal setter/getter methods.
   module NSKVCAccessorUtil
     private
 
     def kvc_internal_setter(key)
-      return '_kvc_internal_' + key.to_s + '=' 
+      return '_kvc_internal_' + key.to_s + '='
     end
 
     def kvc_setter_wrapper(key)
-      return '_kvc_wrapper_' + key.to_s + '=' 
+      return '_kvc_wrapper_' + key.to_s + '='
     end
   end       # module OSX::NSKVCAccessorUtil
 
+  # Interface getter/setter methods from Objecite-C objects via Key-Value Coding.
   module NSKeyValueCodingAttachment
     include NSKVCAccessorUtil
 
-    # invoked from valueForUndefinedKey: of a Cocoa object
+    # Invoked from valueForUndefinedKey: of an Objective-C object
     def rbValueForKey(key)
       if m = kvc_getter_method(key.to_s)
         return send(m)
@@ -488,7 +552,7 @@ module OSX
       end
     end
 
-    # invoked from setValue:forUndefinedKey: of a Cocoa object
+    # Invoked from setValue:forUndefinedKey: of an Objective-C object
     def rbSetValue_forKey(value, key)
       if m = kvc_setter_method(key.to_s)
         send(m, value)
@@ -498,7 +562,7 @@ module OSX
     end
 
     private
-    
+
     # find accesor for key-value coding
     # "key" must be a ruby string
 
@@ -508,7 +572,7 @@ module OSX
       end
       return nil # accessor not found
     end
- 
+
     def kvc_setter_method(key)
       [kvc_internal_setter(key), key + '='].each do |m|
         return m if respond_to? m
@@ -523,13 +587,16 @@ module OSX
 
   end       # module OSX::NSKeyValueCodingAttachment
 
+  # Utility for Key-Value Coding.
   module NSKVCBehaviorAttachment
     include NSKVCAccessorUtil
 
+    # @see #kvc_accessor
     def kvc_reader(*args)
       attr_reader(*args)
     end
 
+    # @see #kvc_accessor
     def kvc_writer(*args)
       args.flatten.each do |key|
 	next if method_defined?(kvc_setter_wrapper(key))
@@ -551,24 +618,28 @@ module OSX
       end
     end
 
+    # Defines accessor methods from given names.
+    # Writer methods aware Key-Value Observing.
     def kvc_accessor(*args)
       kvc_reader(*args)
       kvc_writer(*args)
     end
 
+    # see sample/CurrencyConverter/.
     def kvc_depends_on(keys, *dependencies)
       dependencies.flatten.each do |dependentKey|
         setKeys_triggerChangeNotificationsForDependentKey(Array(keys), dependentKey)
       end
     end
- 
-    # define accesor for keys defined in Cocoa, 
-    # such as NSUserDefaultsController and NSManagedObject
+
+    # Defines accesor methods for keys defined in Objective-C,
+    # such as NSUserDefaultsController and NSManagedObject.
     def kvc_wrapper(*keys)
       kvc_wrapper_reader(*keys)
       kvc_wrapper_writer(*keys)
     end
 
+    # @see #kvc_wrapper
     def kvc_wrapper_reader(*keys)
       keys.flatten.compact.each do |key|
         class_eval <<-EOE_KVC_WRAPPER,__FILE__,__LINE__+1
@@ -579,6 +650,7 @@ module OSX
       end
     end
 
+    # @see #kvc_wrapper
     def kvc_wrapper_writer(*keys)
       keys.flatten.compact.each do |key|
         class_eval <<-EOE_KVC_WRAPPER,__FILE__,__LINE__+1
@@ -592,11 +664,11 @@ module OSX
     # Define accessors that send change notifications for an array.
     # The array instance variable must respond to the following methods:
     #
-    #  length
-    #  [index]
-    #  [index]=
-    #  insert(index,obj)
-    #  delete_at(index)
+    #  - length
+    #  - [index]
+    #  - [index]=
+    #  - insert(index,obj)
+    #  - delete_at(index)
     #
     # Notifications are only sent for accesses through the Cocoa methods:
     #  countOfKey, objectInKeyAtIndex_, insertObject_inKeyAtIndex_,
@@ -660,6 +732,7 @@ module OSX
       end
     end
 
+    private
     # re-wrap at overriding setter method
     def _kvc_behavior_method_added(sym)
       return unless sym.to_s =~ /\A([^=]+)=\z/
@@ -677,7 +750,7 @@ module OSX
   module OCObjWrapper
 
     include NSKeyValueCodingAttachment
-  
+
   end
 
   module OCClsWrapper
@@ -686,10 +759,12 @@ module OSX
     include NSBehaviorAttachment
     include NSKVCBehaviorAttachment
 
+    # Overrides BasicObject#singleton_method_added.
     def singleton_method_added(sym)
       _ns_behavior_method_added(sym, true)
-    end 
- 
+    end
+
+    # Overrides Module#method_added.
     def method_added(sym)
       _ns_behavior_method_added(sym, false)
       _kvc_behavior_method_added(sym)
@@ -697,106 +772,10 @@ module OSX
 
   end
 
+  # Load the foundation frameworks.
+  OSX.load_bridge_support_signatures('CoreFoundation')
+  OSX.load_bridge_support_signatures('Foundation')
+  OSX.ns_import_all
+
 end       # module OSX
 
-# The following code defines a new subclass of Object (Ruby's).
-# 
-#    module OSX 
-#      class NSCocoaClass end 
-#    end
-#
-# This Object.inherited() replaces the subclass of Object class by 
-# a Cocoa class from # OSX.ns_import.
-#
-class Object
-  class <<self
-    def _real_class_and_mod(klass)
-      unless klass.ancestors.include?(OSX::Boxed)
-        klassname = klass.name.to_s
-        unless klassname.nil? || klassname.empty?
-          if Object.included_modules.include?(OSX) and /::/.match(klassname).nil?
-            [klassname, Object]
-          elsif klassname[0..4] == 'OSX::' and (tokens = klassname.split(/::/)).size == 2 and klass.superclass != OSX::Boxed
-            [tokens[1], OSX]
-          end
-        end
-      end
-    end
-
-    alias _before_osx_inherited inherited
-    def inherited(subklass)
-      nsklassname, mod = _real_class_and_mod(subklass) 
-      if nsklassname and (first_char = nsklassname[0]) >= ?A and first_char <= ?Z
-        # remove Ruby's class
-        mod.instance_eval { remove_const nsklassname.intern }
-        begin
-          klass = OSX.ns_import nsklassname.intern
-          raise NameError if klass.nil?
-          subklass = klass
-        rescue NameError
-          # redefine subclass (looks not a Cocoa class)
-          mod.const_set(nsklassname, subklass)
-        end
-      end
-      _before_osx_inherited(subklass)
-    end
-
-    def _register_method(sym, class_method)
-      if self != Object
-        nsklassname, mod = _real_class_and_mod(self)
-        if nsklassname
-          begin
-            nsklass = OSX.const_get(nsklassname)
-            raise NameError unless nsklass.ancestors.include?(OSX::NSObject)
-            if class_method
-              method = self.method(sym).unbind
-              OSX.__rebind_umethod__(nsklass.class, method)
-              nsklass.module_eval do 
-                @__imported_arity = method.arity
-                (class << self; self; end).instance_eval do 
-                  define_method(sym, method)
-                end
-                @__imported_arity = nil
-              end
-            else
-              method = self.instance_method(sym)
-              OSX.__rebind_umethod__(nsklass, method)
-              nsklass.module_eval do
-                @__imported_arity = method.arity
-                define_method(sym, method)
-                @__imported_arity = nil
-              end
-            end
-          rescue NameError
-          end
-        end
-      end
-    end
-
-    alias _before_method_added method_added
-    def method_added(sym)
-      _register_method(sym, false)
-      _before_method_added(sym)
-    end
-
-    alias _before_singleton_method_added singleton_method_added
-    def singleton_method_added(sym)
-      _register_method(sym, true)
-      _before_singleton_method_added(sym)
-    end
-
-    def method_missing(symbol, *args)
-      nsklassname, mod = _real_class_and_mod(self)
-      if nsklassname
-        begin
-          nsklass = OSX.const_get(nsklassname)
-          if nsklass.respond_to?(symbol)
-            return nsklass.send(symbol, *args)
-          end
-        rescue NameError
-        end
-      end
-      raise NoMethodError, "undefined method `#{symbol.to_s}' for #{self}"
-    end
-  end
-end
