@@ -1,4 +1,4 @@
-# $Id: pre-package.rb 2386 2012-09-25 12:45:46Z kimuraw $
+# $Id: pre-package.rb 2520 2014-06-05 14:26:59Z kimuraw $
 
 require 'erb'
 require 'fileutils'
@@ -18,6 +18,7 @@ Dir.mkdir work_dir
 # .plist
 erb('tmpl/Info.plist', File.join(work_dir, 'Info.plist'), binding)
 erb('tmpl/Description.plist', File.join(work_dir, 'Description.plist'), binding)
+erb('tmpl/product.plist', File.join(work_dir, 'product.plist'), binding)
 
 # Resources
 Dir.mkdir resources_dir
@@ -25,23 +26,11 @@ Dir.mkdir File.join(resources_dir, 'English.lproj')
 Dir.mkdir File.join(resources_dir, 'Japanese.lproj')
 
 File.link '../COPYING', File.join(resources_dir, 'License.txt')
-File.link '../ReadMe.html', 
+File.link '../LGPL', File.join(resources_dir, 'LGPL')
+File.link 'tmpl/ReadMe.html', 
           File.join(resources_dir, 'English.lproj', 'ReadMe.html')
-File.link '../ReadMe.ja.html', 
+File.link 'tmpl/ReadMe.ja.html', 
           File.join(resources_dir, 'Japanese.lproj', 'ReadMe.html')
 
 # Contents
 Dir.mkdir contents_dir
-
-# Postflight and post-constent stuff
-if @config['macosx-deployment-target'].to_f < 10.5
-  postflight = File.join(resources_dir, 'postflight')
-  erb('tmpl/postflight-universal.rb', postflight, binding)
-  File.chmod(0755, postflight)
-  usr_lib_dir = File.join(contents_dir, 'usr/lib')
-  FileUtils.mkdir_p usr_lib_dir
-  unless system("tar -xzf ../misc/libruby.1.dylib-tiger.tar.gz -C /tmp")
-    raise "error when decompressing libruby" 
-  end
-  FileUtils.mv '/tmp/libruby.1.dylib', File.join(usr_lib_dir, 'libruby-with-thread-hooks.1.dylib')
-end
