@@ -1,12 +1,12 @@
 #!/usr/bin/env ruby
-# $Id: postflight-universal.rb 2030 2007-09-03 08:11:21Z psychs $
+# $Id: postflight-universal.rb 2412 2013-03-16 07:11:32Z kimuraw $
 # install extlib "rubycocoa.bundle"
 
 require 'rbconfig'
 require 'fileutils'
 
 srcfile = File.join('<%= @config['so-dir'] %>', 'rubycocoa.bundle')
-destdir = Config::CONFIG['sitearchdir']
+destdir = RbConfig::CONFIG['sitearchdir']
 destfile = File.join(destdir, File.basename(srcfile))
 
 begin
