@@ -7,7 +7,7 @@
 //
 
 #import <Cocoa/Cocoa.h>
-#import <RubyCocoa/RubyCocoa.h>
+#import <RubyCocoa/RBRuntime.h>
 
 int main(int argc, const char *argv[])
 {
