@@ -1,9 +1,11 @@
 #!/usr/bin/env ruby
 #
-# $Id: rb_main.rb 2345 2012-08-26 05:18:27Z kimuraw $
+# $Id: rb_main.rb 2592 2014-07-17 15:10:46Z kimuraw $
 #
 
-$KCODE = 'e'
+if RUBY_VERSION < '1.9'
+  $KCODE = 'e'
+end
 require 'hako'
 require 'cocoa_hako'
 
