@@ -11,6 +11,6 @@
 
 int main(int argc, const char *argv[])
 {
-    RBApplicationInit("rb_main.rb", argc, (const char **)argv, nil);
+    RBApplicationInit("console.rb", argc, (const char **)argv, nil);
     return NSApplicationMain(argc, (const char **)argv);
 }
