@@ -2,7 +2,8 @@ require 'test/unit'
 require 'osx/cocoa'
 
 system 'make -s' || raise(RuntimeError, "'make' failed")
-require 'objc_test.bundle'
+require './objc_test.bundle'
+OSX.ns_import :HybridClass if RUBY_VERSION >= '2.0'
 
 class OSX::HybridClass
   def imethod
