@@ -98,6 +98,9 @@ class TC_NSArray < Test::Unit::TestCase
       [(0..4), (0...4), (0..5), (0...5), (1..15),
        (0...-1), (5..3), (3..10), (1..0), (1..-10),
        (-5...3), (-3..-2), (-3...-3)].each do |d|
+	# deletion by nil was removed ruby-1.9 or later
+	next if val.nil? && RUBY_VERSION >= '1.9'
+
         a = alloc_nsarray(1,2,3,4,5)
         b = [1,2,3,4,5]
         a[d] = val
@@ -111,15 +114,24 @@ class TC_NSArray < Test::Unit::TestCase
     a = alloc_nsarray(1,2,3,4,5)
     assert_raise(RangeError) { a[-10..5] = nil }
     assert_raise(RangeError) { a[10..5] = nil }
-    assert_nothing_raised { a[5..5] = nil }
-    assert_nothing_raised { a[5..-10] = nil }
-    assert_nothing_raised { a[-5..10] = nil }
+    if RUBY_VERSION >= '2.0'
+      assert_raise(ArgumentError) { a[5..5] = nil }
+      assert_raise(ArgumentError) { a[5..-10] = nil }
+      assert_raise(ArgumentError) { a[-5..10] = nil }
+    else
+      assert_nothing_raised { a[5..5] = nil }
+      assert_nothing_raised { a[5..-10] = nil }
+      assert_nothing_raised { a[-5..10] = nil }
+    end
   end
 
   def test_assign_start_and_length
     [33, nil, [], [11,22,33], NSArray.arrayWithArray([1,2,3])].each do |val|
       [[1,3], [1,10], [-3,2], [-3,10], [-3,0],
        [-1,0], [0,0],[-2,3], [-2,2]].each do |d|
+	# deletion by nil was removed ruby-1.9 or later
+	next if val.nil? && RUBY_VERSION >= '1.9'
+
         a = alloc_nsarray(1,2,3,4,5)
         b = [1,2,3,4,5]
         a[*d] = val
@@ -131,10 +143,10 @@ class TC_NSArray < Test::Unit::TestCase
 
   def test_assign_start_and_length_error
     a = alloc_nsarray(1,2,3,4,5)
-    assert_raise(IndexError) { a[-10,2] = nil }
-    assert_raise(IndexError) { a[10,2] = nil }
-    assert_raise(IndexError) { a[3,-2] = nil }
-    assert_nothing_raised { a[3,0] = nil }
+    assert_raise(IndexError) { a[-10,2] = 0 }
+    assert_raise(IndexError) { a[10,2] = 0 }
+    assert_raise(IndexError) { a[3,-2] = 0 }
+    assert_nothing_raised { a[3,0] = 0 }
   end
 
   def test_plus
@@ -864,6 +876,10 @@ class TC_NSArray < Test::Unit::TestCase
   end
   
   def test_grep
+    assert_equal(true, /a/ === 'abc')
+    assert_equal(true, /a/ === 'abc'.to_ns)
+    assert_equal(false, /a/ === 1)
+    assert_equal(false, /a/ === 1.to_ns)
     [['abc','d!ef','98adfa','zAvcz'], [1,2,3,4]].each do |d|
       a = alloc_nsarray(*d)
       b = d
