@@ -2,12 +2,28 @@
 require 'test/unit'
 require 'osx/cocoa'
 
-def with_kcode(k)
-  cur = $KCODE
-  $KCODE = k
-  yield
-ensure
-  $KCODE = cur
+if RUBY_VERSION >= '2.0'
+  def with_kcode(k)
+    cur = Encoding.default_internal
+    enc = case k
+    when 'utf-8'
+      Encoding::UTF_8
+    else
+      raise ArgumentError
+    end
+    Encoding.default_internal = enc
+    yield
+  ensure
+    Encoding.default_internal = cur
+  end
+else
+  def with_kcode(k)
+    cur = $KCODE
+    $KCODE = k
+    yield
+  ensure
+    $KCODE = cur
+  end
 end
 
 class TC_ObjcString < Test::Unit::TestCase
@@ -173,8 +189,13 @@ class TC_ObjcString < Test::Unit::TestCase
     
     with_kcode('utf-8') do
       s = alloc_nsstring('foo かきくけこ')
-      assert_equal(0x3051, s[-2])
-      assert_equal(0x3053, s[8])
+      if RUBY_VERSION >= '2.0'
+	assert_equal("\u3051", s[-2])
+	assert_equal("\u3053", s[8])
+      else
+	assert_equal(0x3051, s[-2])
+	assert_equal(0x3053, s[8])
+      end
       assert_equal(nil, s[10])
       assert_equal(nil, s[-10])
     end
@@ -240,14 +261,25 @@ class TC_ObjcString < Test::Unit::TestCase
   def test_assign_nth_error
     s = 'abc'
     n = alloc_nsstring(s)
-    assert_raise(IndexError) { s[3] = '' }
-    assert_raise(IndexError) { n[3] = '' }
+    # ruby-2.0 or later, str[len]=other appends other to str.
+    if RUBY_VERSION >= '2.0'
+      assert_nothing_raised { s[3] = '' }
+      assert_nothing_raised { n[3] = '' }
+    else
+      assert_raise(IndexError) { s[3] = '' }
+      assert_raise(IndexError) { n[3] = '' }
+    end
     assert_raise(IndexError) { s[-4] = '' }
     assert_raise(IndexError) { n[-4] = '' }
     s = ''
     n = alloc_nsstring(s)
-    assert_raise(IndexError) { s[0] = '' }
-    assert_raise(IndexError) { n[0] = '' }
+    if RUBY_VERSION >= '2.0'
+      assert_nothing_raised { s[0] = '' }
+      assert_nothing_raised { n[0] = '' }
+    else
+      assert_raise(IndexError) { s[0] = '' }
+      assert_raise(IndexError) { n[0] = '' }
+    end
     assert_raise(IndexError) { s[-1] = '' }
     assert_raise(IndexError) { n[-1] = '' }
   end
@@ -535,14 +567,18 @@ class TC_ObjcString < Test::Unit::TestCase
     with_kcode('utf-8') do
       s = alloc_nsstring('abc def')
       assert_equal(true, s.include?('c d'))
-      assert_equal(true, s.include?(0x62))
       assert_equal(false, s.include?('C D'))
-      assert_equal(false, s.include?(0x41))
+      if RUBY_VERSION < '2.0'
+	assert_equal(true, s.include?(0x62))
+	assert_equal(false, s.include?(0x41))
+      end
       s = alloc_nsstring('abc かきくけこ')
       assert_equal(true, s.include?('かき'))
-      assert_equal(true, s.include?(0x3053))
       assert_equal(false, s.include?('は'))
-      assert_equal(false, s.include?(0x3060))
+      if RUBY_VERSION < '2.0'
+	assert_equal(true, s.include?(0x3053))
+	assert_equal(false, s.include?(0x3060))
+      end
     end
   end
   
@@ -551,10 +587,18 @@ class TC_ObjcString < Test::Unit::TestCase
     n = alloc_nsstring(s)
     assert_raise(TypeError) { s.include?([]) }
     assert_raise(TypeError) { n.include?([]) }
+    if RUBY_VERSION >= '2.0'
+      assert_raise(TypeError) { s.include?(0x62) }
+      assert_raise(TypeError) { s.include?(0x3053) }
+    end
   end
   
   def test_index
     ['', 'a', 'z', 0x42, 0x100000].each do |i|
+      # ruby-2.0 or later, String#index() does not accept
+      if (i.kind_of?(Fixnum) && RUBY_VERSION >= '2.0')
+	next
+      end
       s = 'abc'
       n = alloc_nsstring(s)
       assert_equal(s.index(i), n.index(i))
@@ -571,6 +615,11 @@ class TC_ObjcString < Test::Unit::TestCase
     n = alloc_nsstring(s)
     assert_raise(TypeError) { s.index([]) }
     assert_raise(TypeError) { n.index([]) }
+    # ruby-2.0 or later, String#index() does not accept
+    if RUBY_VERSION >= '2.0'
+      assert_raise(TypeError) { s.index(0) }
+      assert_raise(TypeError) { n.index(0) }
+    end
   end
   
   def test_insert
@@ -610,7 +659,7 @@ class TC_ObjcString < Test::Unit::TestCase
      "\nabc\n\ndef\nghi\njkl\n\nmnopq\n\n"].each do |s|
       n = alloc_nsstring(s)
       a = []
-      s.each {|i| a << i }
+      s.each_line {|i| a << i }
       assert_equal(n.lines, a)
       assert_kind_of(NSArray, n.lines)
     end
@@ -663,9 +712,18 @@ class TC_ObjcString < Test::Unit::TestCase
   end
   
   def test_ord
-    ['', 'a', 'Z', '0', "\n"].each do |s|
+    ['a', 'Z', '0', "\n"].each do |s|
       n = alloc_nsstring(s)
-      assert_equal(s[0] || 0, n.ord)
+      o = if s.respond_to?(:ord)
+	s.ord
+      else
+	s[0] # ruby-1.8
+      end
+      assert_equal(o, n.ord)
+    end
+    ['', alloc_nsstring('')].each do |s|
+      next unless s.respond_to?(:ord)
+      assert_raise(ArgumentError) {s.ord}
     end
   end
   
@@ -704,7 +762,13 @@ class TC_ObjcString < Test::Unit::TestCase
     ['', 'a', 'z', 0x42, 0x100000].each do |i|
       s = 'abcabc'
       n = alloc_nsstring(s)
-      assert_equal(s.rindex(i), n.rindex(i))
+      # ruby-2.0 or later, String#index() does not accept
+      if (i.kind_of?(Fixnum) && RUBY_VERSION >= '2.0')
+	assert_raise(TypeError) { s.rindex(i) }
+	assert_raise(TypeError) { n.rindex(i) }
+      else
+	assert_equal(s.rindex(i), n.rindex(i))
+      end
     end
     [-10,-6,-5,-4,-3,-2,-1,0,1,2,3,4,5,6,10].each do |i|
       s = 'abcabc'
@@ -950,7 +1014,12 @@ class TC_ObjcString < Test::Unit::TestCase
     b = alloc_nsstring('ba')
     r = []
     a.upto(b) {|i| r << i }
-    r.map! {|i| i.to_ruby }
+    if RUBY_VERSION >= '2.0'
+      # Range#each yields String when its element responds to :to_str on ruby-2.0 or later
+    else
+      r.map! {|i| i.to_ruby }
+    end
+    assert_kind_of(String, r.first)
     c = 'aa'
     d = 'ba'
     s = []
