@@ -6,8 +6,8 @@ rescue LoadError
 end
 require 'test/unit'
 
-`ls tc_*.rb`.each do |testcase|
+`ls tc_*.rb`.each_line do |testcase|
   testcase.chop!
   $stderr.puts testcase if $VERBOSE
-  require( testcase )
+  require(File.join('.', testcase ))
 end
